/**
 * These classes are implementing the data store of the corpus_analysis
 * package. If we follow the idea of the Model View Controller Pattern these
 * classes combined in the Results class define the Model.
 */

class Results {
  constructor() {
  this.data = new Data();
  this.metaData = new MetaData();
  this.fullResultsData = new Data();
  this.subResultsData = new Data();
  this.inspectResultsData = new Data();
  console.info('Initialized the Results object.');
  }

  init() {
    this.data.init();
    this.metaData.init();
    this.fullResultsData.init();
    this.subResultsData.init();
    this.inspectResultsData.init();
  }

}

class Data {
  // Sets empty object structure. Also usefull to delete old results.
  // matchCount default is 0
  init(matchCount=0, type="results") {
    this.matches = [];  // list of all c with lc and rc
    this.cpos_lookup = {};  // object contains all this key value pair
    this.text_lookup = {};  // same as above for all text ids
    this.match_count = matchCount;
    this.corpus_type = 'results';
    this.cpos_ranges = null;
    this.query = '';
  }

  addData(jsonData, key=null) {
    if (key !== null) {
      Object.assign(this[key], jsonData);
    } else if (key === null) {
      Object.assign(this, jsonData)
    }
  }

  // get query as string from form Element
  getQueryStr(queryFormElement) {
    // gets query
    let queryFormData;
    let queryStr;
    queryFormData = new FormData(queryFormElement);
    queryStr = queryFormData.get('query-form-query');
    this['query'] = queryStr;
  }

  // function creates a unique and safe filename for the download
  createDownloadFilename(suffix) {
    let today = new Date();
    let currentDate = `${today.getUTCFullYear()}` +
                      `-${(today.getUTCMonth() + 1)}` +
                      `-${today.getUTCDate()}`;
    let currentTime = `${today.getUTCHours()}h` +
                      `${today.getUTCMinutes()}m` +
                      `${today.getUTCSeconds()}s`;
    let safeFilename = this.query.replace(/[^a-z0-9_-]/gi, "_");
    let resultFilename = `UTC-${currentDate}_${currentTime}_${safeFilename}_${suffix}`;
    return resultFilename
  }
  /**
   * Function to download data as Blob created from string.
   * Should be private but that is not yet a feature of javascript 08.04.2020
   */
  download(downloadElement, dataStr, filename, type, filenameSlug) {
    filename += filenameSlug;
    let file = new Blob([dataStr], {type: type});
        var url = URL.createObjectURL(file);
        downloadElement.href = url;
        downloadElement.download = filename;
  }

  // function to download the results as JSON
  downloadJSONRessource(resultFilename, downloadData, downloadElement) {
    /**
     * Stringify JSON object for json download.
     * Use tabs to save some space.
     */
    let dataStr = JSON.stringify(downloadData, undefined, "\t");
    // Start actual download
    this.download(downloadElement, dataStr, resultFilename, "text/json", ".json")
  }

}

class MetaData {
  // Sets empty object structure when no input is given.
  // if json object like input is given class fields are created from this
  init(json={}) {
    Object.assign(this, json);
  }
}

export {Results, Data, MetaData};